//import javax.swing.*;
import java.awt.*;
import java.util.Vector;

public class AnimationPanel extends java.awt.Panel {

  public AnimationPanel(int _id, int _nProcCol, int _nProcRow, int _matrixCol, int _matrixRow, Color _colorBackground, Color _colorBoundary, Color _colorBuffer, Color _colorLocalOld, Color _colorLocalNew, Color _colorLocalNewChanged, Matrix _parent, Laplace _appletMain) {

    myID = _id;
    nProcCol = _nProcCol;
    nProcRow = _nProcRow;
    matrixCol = _matrixCol;
    matrixRow = _matrixRow;
    parent = _parent;


    colorBackground = _colorBackground;
    colorBoundary   = _colorBoundary;
    colorBuffer     = _colorBuffer;
    colorLocalOld   = _colorLocalOld;
    colorLocalNew   = _colorLocalNew;
    colorLocalNewChanged = _colorLocalNewChanged;

    appletMain = _appletMain;

    infoSetting();

    setupGUI();
    //updateData();
    //updateMatrix();
    //animationThread = new Thread(this);
  }

  protected void infoSetting() {
    localCol = matrixCol / nProcCol;
    localRow = matrixRow / nProcRow;

    procX = (int)(myID % nProcCol);
    procY = (int)(myID / nProcCol);

    startRowIndex = 0;
    startColIndex = 0;

    endRowIndex = localRow;
    endColIndex = localCol;

    if ( procY == 0 ) {
      northNeighborID = -1;
      northNeighbor = false;
      startRowIndex++;
    }
    else {
      northNeighborID =  procX + (procY-1) * nProcCol;
      northNeighbor = true;
    }

    if ( procY == nProcRow-1 ) {
      southNeighborID = -1;
      southNeighbor = false;
      endRowIndex--;
    }
    else {
      southNeighborID = procX + (procY+1) * nProcCol;
      southNeighbor = true;
    }

    if ( procX == nProcCol-1 ) {
      eastNeighborID = -1;
      eastNeighbor = false;
      endColIndex--;
    }
    else {
      eastNeighborID = procX + 1 + (procY * nProcCol);
      eastNeighbor = true;
    }

    if ( procX == 0 ) {
      westNeighborID = -1;
      westNeighbor = false;
      startColIndex++;
    }
    else {
      westNeighborID = procX - 1 + (procY * nProcCol);
      westNeighbor = true;
    }

    // gettinig the matrix data
    double originalData[][];
    originalData = new double[matrixCol][matrixRow];
    originalData = parent.getMatrix();
    matrixData = new double[localCol+2][localRow+2];
    newMatrixData = new double[localCol+2][localRow+2];

    for ( int i = 0 ; i < localRow ; i++ ) {
      int tmpRow = procY * localRow + i;
      for ( int j = 0 ; j < localCol ; j++ ) {
	int tmpCol = procX * localCol + j;
	matrixData[j+1][i+1] = originalData[tmpCol][tmpRow];
	newMatrixData[j+1][i+1] = originalData[tmpCol][tmpRow];
      }
    }

  }

  protected void redoGUI() {
    removeAll();
    setupGUI();
  }

  protected void setupGUI() {        

    setLayout(new BorderLayout(5, 5));

    // Local Matrix
    processMain = new PartitionPanel(myID, nProcCol, nProcRow, matrixCol, matrixRow, colorBackground, colorBoundary, colorBuffer, colorLocalOld, colorLocalNew, colorLocalNewChanged, parent, this);
    add("Center", processMain);

    // north animation
    if ( northNeighbor ) {
      northAnimate = new AnimationCanvas(colorBackground, 4, appletMain);
      northAnimate.redraw(false);
      add("North", northAnimate);
    }

    // south animation
    if ( southNeighbor ) {
      southAnimate = new AnimationCanvas(colorBackground, 3, appletMain);
      southAnimate.redraw(false);
      add("South", southAnimate);
    }

    // west animation
    if ( westNeighbor ) {
      westAnimate = new AnimationCanvas(colorBackground, 2, appletMain, northNeighbor);
      westAnimate.redraw(false);
      add("West", westAnimate);
    }

    // east animation
    if ( eastNeighbor ) {
      eastAnimate = new AnimationCanvas(colorBackground, 1, appletMain, northNeighbor);
      eastAnimate.redraw(false);
      add("East", eastAnimate);
    }

  }

  public void startProcessor() {
    //updateData();
    processMain.startProcessor();
  }

  public void suspendProcessor() {
    processMain.suspendProcessor();
  }

  public void resumeProcessor() {
    processMain.resumeProcessor();
  }

  public void nextStepProcessor() {
    processMain.nextStepProcessor();
  }

  public void setDemoSpeed(int speed) {
    processMain.setDemoSpeed(speed);
  }


  protected int myID = 0;
  protected int nProcCol = 0;
  protected int nProcRow = 0;
  protected int matrixCol = 0;
  protected int matrixRow = 0;
  protected Matrix parent = null;
  protected Laplace appletMain = null;

  protected int localCol = 0;
  protected int localRow = 0;

  protected int procX = 0;
  protected int procY = 0;

  protected boolean northNeighbor = true;
  protected boolean southNeighbor = true;
  protected boolean eastNeighbor = true;
  protected boolean westNeighbor = true;

  protected Color colorBackground;
  protected Color colorBoundary;
  protected Color colorBuffer;
  protected Color colorLocalOld;
  protected Color colorLocalNew;
  protected Color colorLocalNewChanged;

  protected int northNeighborID = -1;
  protected int southNeighborID = -1;
  protected int eastNeighborID = -1;
  protected int westNeighborID = -1;

  protected PartitionPanel processMain;
  protected RowBuffer northBuffer;
  protected RowBuffer southBuffer;
  protected ColumnBuffer eastBuffer;
  protected ColumnBuffer westBuffer;

  protected double matrixData[][];
  protected double newMatrixData[][];

  protected int startRowIndex = 0;
  protected int startColIndex = 0;

  protected int endRowIndex = 0;
  protected int endColIndex = 0;

  protected AnimationCanvas northAnimate;
  protected AnimationCanvas southAnimate;
  protected AnimationCanvas eastAnimate;
  protected AnimationCanvas westAnimate;

  public Thread animationThread;
}
